<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * MA Module
 */
class MarketingAutomationModule extends CWebModule
{
    
    /**
     * Determines if the MarkaTrackingWidget is enabled 
     * (if false, the tracking widget will not render anything and thus the 
     * Marketing Automation tracker will not be contacted by the client).
     * @var boolean 
     */
    public $markaTrackingEnabled = true;
    
    /**
     * Determines if the MarkaSuggestionsWidget is enabled 
     * (if false, the suggestions widget will not render anything and will not
     * contact the Marketing Automation tracker).
     * @var boolean 
     */
    public $markaSuggestionsEnabled = true;
    
    /**
     * The base URL of the Marketing Automation tracker.
     * @var string
     */
    public $markaTrackerBaseUrl = 'http://localhost:8081';
	
    /**
     * The name of the cookie, where the user ID used by Marketing Automation is stored.
     * @var string
     */
	public $markaCookieName = 'markaCookie';
    
    /**
     * The number of seconds, after which cached suggested page infos will expire.
     * Default: 10 minutes
     * @var int 
     */
    public $cacheTime = 600;
    
    /**
     * An optional array of page attributes, which should be retrieved for each
     * suggested page and passed on to the view in addition to 'name' and 'url'.
     * Note that these attributes are CMS attributes like 'filename' or 
     * 'publishdate', not Marketing Automation attributes. For example:
     * array('folder_id', 'edittimestamp')
     * A value of NULL means that only 'name' and 'url' will be retrieved.
     * @var array 
     */
    public $suggestionsAdditionalPageAttributes = NULL;

    /**
     * This array is used to configure which iMarkaPageInfosProvider should be used to obtain page infos
     * (for the suggestions) for which Marketing Automation page ID prefix. The entries in the array have to
     * be of the form 'markaIdPrefix' => 'path.of.the.PageInfosProviderClass'
     * Each provider must implement the common.modules.marketingautomation.components.iMarkaPageInfosProvider interface.
     * Note that there can be a problem if there is a prefix, which is itself  
     * a prefix of another prefix, e.g., prefix1: 'a', prefix2: 'ab'. In this case all pageIds that start with
     * 'a' will be passed to the provider for prefix1. To make sure that this scenario works correctly, the provider
     * for prefix2 has to be placed in the array before the provider for prefix1.
     * An example can be found in frontend/config/main.php in the config for the marketingautomation module.
     * @var array
     */
    public $pageInfosProviders;

    /**
     * init
     *
     * @return void
     */
    public function init()
    {
        parent::init();
    }
    
    /**
     * Public getter for determining if the MarkaTrackingWidget is enabled 
     * (if false, the tracking widget will not render anything and thus the 
     * Marketing Automation tracker will not be contacted by the client).
     * @return boolean 
     */
    public function isMarkaTrackingEnabled()
    {
        return $this->markaTrackingEnabled;
    }
    
    /**
     * Public getter for determining if the MarkaSuggestionsWidget is enabled 
     * (if false, the suggestions widget will not render anything and will not
     * contact the Marketing Automation tracker).
     * @return boolean 
     */
    public function isMarkaSuggestionsEnabled()
    {
        return $this->markaSuggestionsEnabled;
    }
    
    /**
     * Public getter for the base URL of the Marketing Automation tracker.
     * @return string
     */
    public function getMarkaTrackerBaseUrl()
    {
        return $this->markaTrackerBaseUrl;
    }

    /**
     * Gets the associative array of iMarkaPageInfosProvider instances.
     * @return array
     */
    public function getPageInfosProviders()
    {
        $instances = array();
        foreach ($this->pageInfosProviders as $prefix => $classPath) {
            $indexOfLastDot = strrpos($classPath, '.');
            if ($indexOfLastDot === FALSE || $indexOfLastDot === strlen($classPath)) {
                die('Error in the marketingautomation module configuration. Could not determine the class name of the iMarkaPageInfosProvider ' . $classPath);
            }
            $className = substr($classPath, $indexOfLastDot + 1);
            Yii::import($classPath);
            $instances[$prefix] = new $className();
        }
        return $instances;
    }
	
    /**
     * Gets the UUID for the user, which identifies the user for Marketing Automation.
     * If the user is logged in, the UUID is read from his profile (and saved to
     * the cookie if it has not been saved there yet), if the user is not logged
     * in, it is read from the cookie.
     * If the user does not have a Marketing Automation ID yet, a new one will 
     * be generated and saved to the cookie and the user's profile, if he is logged in.
     * @return string
     */
	public function getUUID()
    {
		$uuid = '';
		//check if user is logged in and read markaUUID from profile
		//if nothing is set save current UUID or generate new one
		if (!Yii::app()->user->isGuest) {					
			$uuid = $this->_getOrSaveUUIDFromProfile();			
		}
		//if no uuid is set, read it from cookie or generate new one
		if ($uuid == '') {			
			$cookieUUID = isset(Yii::app()->request->cookies[$this->markaCookieName]) ? Yii::app()->request->cookies[$this->markaCookieName]->value : '';
			if($cookieUUID != ''){
				$uuid = $cookieUUID;
			} else{
				$uuid = $this->_generateUUIDAndSaveToCookie();
			}
		}
		return $uuid;
	}
	
    /**
     * Generates a new UUID, saves it to the cookie and then returns it.
     * @return string
     */
	private function _generateUUIDAndSaveToCookie()
    {
		$uuid = uniqid('',true);
		
		$cookie = new CHttpCookie($this->markaCookieName, $uuid);
		//set cookie
		$cookie->expire = time()+60*60*24*3650; 
		Yii::app()->request->cookies[$this->markaCookieName] = $cookie;
		
		return $uuid;
	}
	
    /**
     * Gets the user's UUID from his profile or generates a new one (and saves
     * it to the profile and the cookie) if the user does not have a Marketing Automation ID yet.
     * @return string
     */
	private function _getOrSaveUUIDFromProfile()
    {
		$uuid = '';
		$profile = Profile::model()->findByAttributes(array('user_id' => Yii::app()->user->id));
		if ($profile) {
			if (isset($profile->markaUUID) && $profile->markaUUID != '') {
				$uuid = $profile->markaUUID;
			}
		}
		if ($uuid == '') {
			//marka UUID not saved yet - save it from current cookie or generate new one
			$cookieUUID = isset(Yii::app()->request->cookies[$this->markaCookieName]) ? Yii::app()->request->cookies[$this->markaCookieName]->value : '';
			if ($cookieUUID != '') {
				$uuid = $cookieUUID;
				$profile->markaUUID = $cookieUUID;
				$profile->save();
			} else{
				$uuid = $this->_generateUUIDAndSaveToCookie();
				$profile->markaUUID = $uuid;
				$profile->save();
			}
		}		
		return $uuid;
	}
}
