<?php
/**
 * Class for compressing and extracting set of folders/files in zip format
 */
class ZipHelper
{
    /**
     * Recursively zip folder
     *
     * @param array  $sources     array of sources. String key means source contains string of content.
     *                            Otherwise source contains name of file
     * @param string $destination zip archive destination folder
     *
     * @throws Exception
     * @return bool
     */
    public static function compress($sources, $destination)
    {
        if (!extension_loaded('zip')) {
            throw new Exception('Php Zip extension not loaded');
        }
        if (empty($sources)) {
            return false;
        }
        if (is_string($sources)) {
            $sources = array($sources);
        }
        $zip = new ZipArchive();
        if (!$zip->open($destination, ZIPARCHIVE::CREATE | ZIPARCHIVE::OVERWRITE | ZIPARCHIVE::CHECKCONS)) {
            return false;
        }
        foreach ($sources as $name => $source) {
            if (is_int($name)) {
                $exclude = array();
                $sourcePath = $source;
                if (is_array($source)) {
                    $sourcePath = $source['path'];
                    if (isset($source['exclude'])) {
                        $exclude = $source['exclude'];
                    }
                }
                $sourcePath = str_replace('\\', '/', realpath($sourcePath));
                if (is_dir($sourcePath) === true) {
                    $iterator = new RecursiveIteratorIterator(
                        new RecursiveDirectoryIterator($sourcePath, RecursiveDirectoryIterator::SKIP_DOTS),
                        RecursiveIteratorIterator::SELF_FIRST
                    );
                    foreach ($iterator as $file) {
                        if ($file->isLink()) {
                            continue;
                        }
                        //for windows systems
                        $file = str_replace('\\', '/', realpath($file));
                        $zipPath = str_replace($sourcePath . '/', '', $file);
                        //if in $exclude skip
                        $skip = count(
                            array_filter(
                                $exclude,
                                function($ex) use ($zipPath)
                                {
                                    return strpos($zipPath, $ex) === 0;
                                }
                            )
                        ) > 0;
                        if ($skip) {
                            continue;
                        }
                        //add to archive
                        if (is_dir($file) === true) {
                            $zip->addEmptyDir($zipPath . '/');
                        } else if (is_file($file) === true) {
                            $zip->addFromString($zipPath, file_get_contents($file));
                        }
                    }
                } else if (is_file($sourcePath) === true) {
                    $zip->addFromString(basename($sourcePath), file_get_contents($sourcePath));
                }
            } else {
                $zip->addFromString($name, $source);
            }
        }
        return $zip->close();
    }

    /**
     * Extract zip archive to folder
     *
     * @param string $zipPath path to zip archive
     * @param string $dest    destination folder
     * @param bool   $merge   if false replace destination folder by new folders from archive, otherwise merge folders
     *
     * @throws Exception
     * @return bool true if extraction success, false otherwise
     */
    public static function extract($zipPath, $dest, $merge = true)
    {
        if (!extension_loaded('zip')) {
            throw new Exception('Php Zip extension not loaded');
        }
        $zipH = zip_open($zipPath);
        if (!$zipH) {
            throw new Exception("Can't open archive $zipPath");
        }
        if (file_exists($dest) && !is_writable($dest)) {
            throw new Exception("Destination directory $dest is not writable");
        } elseif (!is_writable($dest) && !mkdir($dest)) {
            throw new Exception("Can't create directory in $dest");
        }

        while ($entry = zip_read($zipH)) {
            $entryPath = zip_entry_name($entry);
            $destPath = $dest . '/' . $entryPath;
            if (substr($entryPath, -1) == '/') {
                if (file_exists($destPath)) {
                    if (!$merge) {
                        FileHelper::rmdirr($destPath);
                        mkdir($destPath);
                    }
                } else {
                    mkdir($destPath);
                }
            } else {
                if (!is_dir($dir = dirname($destPath))) {
                    mkdir($dir, 0777, true);
                }
                if (@file_put_contents($destPath, zip_entry_read($entry, zip_entry_filesize($entry))) === false) {
                    throw new Exception("$destPath is not writable");
                }
            }
        }
    }
}
