#!/usr/bin/env php
<?php
/**
 * Gentics Portal.Node PHP - Comamnd Line Tool
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 */



/**************************
 *       Functions
 **************************/

/**
 * Prints a message out with a linebreak at the end
 *
 * @param string $text The text to print
 */
function printLine($text)
{
	echo $text . "\n";
}

/**
 * Prompts the user with a question where he can answer with "yes" or "no"
 *
 * @param string  $text                       The question to ask
 * @param boolean $defaultYes                 Default answer if the user just presses return, can be either true for yes or false for no
 * @param string  $unattendedInstallationName Keyname for unattended installation (for command line arguments)
 * @return string true for yes or false for no
 */
function prompt($question, $defaultYes=true, $unattendedInstallationName="")
{
	$options = &$GLOBALS['options'];

	// If this option was passed as argument, use that instead
	if (!empty($options[$unattendedInstallationName])) {
		return $options[$unattendedInstallationName];
	}

	// If unattended installation is activated, return the default answer
	if (isset($options['unattended-installation'])) {
		return $defaultYes;
	}

	// Set possible options
	if ($defaultYes) {
		$options = "[ " . ansiColor('purple') . "Yes" . ansiColor(null) . " | No ]";
	} else {
		$options = "[ Yes | " . ansiColor('purple') . "No" . ansiColor(null) . " ]";
	}

	$answer = ask($question . ansiColor(null) . "\n" . $options, null, true);

	if (yes($answer)) {
		return true;
	} elseif (no($answer)) {
		return false;
	} elseif (empty($answer)) {
		return $defaultYes;
	} else {
		return prompt($question);
	}
}

/**
 * Ask the user a question and let him answer with any answer
 *
 * @param string $text          The question to ask the user
 * @param array  $options       An array of options where they keys are the possible inputs and the values are the informational texts
 * @param bool   $allowEmpty    Whether an empty answer should be valid or not (if not it will repeat the question automatically)
 * @param string $defaultAnswer The default answer for unattended installation
 */
function ask($text, $options=null, $allowEmpty=false, $defaultAnswer="")
{
	// If unattended installation is activated, return the default answer
	if (isset($GLOBALS['options']['unattended-installation'])) {
		if ($allowEmpty || !empty($defaultAnswer)) {
			return $defaultAnswer;
		}
	}

	while (true) {
		echo ansiColor('yellow') . $text . ansiColor(null) . "\n";

		if (is_array($options)) {
			foreach ($options as $key => $option) {
				echo ansiColor('purple') . $key . ansiColor(null) . ") " . $option . "\n";
			}
		}

		$answer = trim(fgets(STDIN));
		echo "\n";

		if ($allowEmpty || !empty($answer)) {
			break;
		}
	}

	return $answer;
}

/**
 * Return true if 'y' was passed or false otherwise
 *
 * @param string $string 'y' or any other string
 * @return boolean See function description
 */
function yes($string)
{
	return strtolower(substr($string, 0, 1)) === 'y';
}

/**
 * Return true if 'n' was passed or false otherwise
 *
 * @param string $string 'n' or any other string
 * @return boolean See function description
 */
function no($string)
{
	return strtolower(substr($string, 0, 1)) === 'n';
}

/**
 * Checks if the passed returnCode is an error and outputs $errorMessage if yes
 *
 * @param integer A code returned by a binary/script
 * @param string  An error message
 */
function handleError($returnCode, $errorMessage)
{
	if ($returnCode !== 0) {
		echo "\n\nERROR: " . $errorMessage;
		echo "Aborting with errorcode " . $returnCode . "\n\n";
		exit(10);
	}
}

/**
 * Check if the current code is executed on Windows
 *
 * @return boolean True if windows, false otherwise.
 */
function isWindows()
{
	return strtoupper(substr(PHP_OS, 0, 3)) === 'WIN';
}

/**
 * Check if the given $command exists in any directory of the current
 * PATH environment variable
 *
 * @param string $command Command to check
 * @return boolean True if the command is available, false otherwise.
 */
function shellCommandExists($command)
{
	$whereIsCommand = null;
	if (isWindows()) {
		$whereIsCommand = 'where ' . escapeshellarg($command) . ' 2> nul';
	} else {
		$whereIsCommand = 'which ' . escapeshellarg($command) . ' 2> /dev/null';
	}

	$output = shell_exec($whereIsCommand);
	return !empty($output);
}

/**
 * Copy the contents of $sourceDirectory to $targetDirectory
 * Create $targetDirectory automatically with mode 0755 if it does not exist.
 *
 * @param string $sourceDirectory
 * @param string $targetDirectory
 * @return boolean True on success, false otherwise
 */
function copyDirectoryContentsRecursively($sourceDirectory, $targetDirectory)
{
	if (!file_exists($targetDirectory) && !mkdir($targetDirectory, 0755, true)) {
		printLine(ansiColor('red') . "[ERROR] Could not create directory " . $targetDirectory . ansiColor(null));
		return false;
	}

	if (isWindows()) {
		echo shell_exec("xcopy \"" . $sourceDirectory . "/\"* \"" . $targetDirectory . "\" /s /e /h /-y");
	} else {
		echo shell_exec("cp -R \"" . $sourceDirectory . "/\"* \"" . $targetDirectory . "\"");
	}

	return true;
}

/**
 * Delete $directory
 *
 * @param string $directory
 */
function removeDirectoryRecursively($directory)
{
	if (isWindows()) {
		echo shell_exec("del \"" . $directory . "\" /s");
	} else {
		echo shell_exec("rm -R \"" . $directory . "\"");
	}
}

/**
 * Run the yii database migration tool
 *
 * @param string $yiicTool Path to the yii tool (can be relative to the current working directory)
 */
function runYiiMigration($yiicTool)
{
	passthru($yiicTool . " migrate --interactive=0");
}

/**
 * Output the specified color as ANSI color.
 * The colors are defined in the global array $ansiColors.
 * If the color can't be found, it is reset.
 *
 * @param string  $color Color string, must be defined in the global $ansiColors
 * @return string A Ansi escape code
 */
function ansiColor($color)
{
	if (isWindows()) {
		// Todo: Find out how to output color escape sequences in Git Bash on Windows
		// Symfony2 is able to do this somehow.
		return "";
	}

	$ansiColorCode = null;
	if (empty($color) || !isset($GLOBALS['ansiColors'][$color])) {
		$ansiColorCode = "0";
	} else {
		$ansiColorCode = $GLOBALS['ansiColors'][$color];
	}

	return "\033[" . $ansiColorCode . "m";
}

/**
 * Install Gentics Portal.Node PHP
 */
function install()
{
	if (!isWindows() && posix_geteuid() > 0) {
		printLine(ansiColor('red') . "[ERROR] The installation can only be run by root" . ansiColor(null) . "\n");
		exit();
	}

	$settings = array();

	printLine(ansiColor('green') . "# This installer will install Gentics Portal.Node PHP" . ansiColor(null) . "\n");

	if (isset($GLOBALS['options']['install-lamp']) && !isWindows()) {
		printLine("Installing a LAMP-system...");

		passthru("apt-get --assume-yes install nginx");
		// MySQL:
		passthru("apt-get --assume-yes install mysql-server mysql-client");
		// PHP5 with modules:
		passthru("apt-get --assume-yes install php5 libapache2-mod-php5 php5-mysql php5-gd");
		// APC, MemCached:
		passthru("apt-get --assume-yes install memcached php5-memcache php5-memcached");
		passthru("apt-get --assume-yes install php-pear php5-dev make");
		passthru("pecl install apc");
	}

	if (ENVIRONMENT === "build") {
		// Copy the GPN PHP application
		$newRootDirectory = null;
		while (strlen($newRootDirectory) <= 1) {
			$newRootDirectory = ask("Where do you want to install Gentics Portal.Node PHP to?\n"
				. "- " . ansiColor('red') . "Warning: this will overwrite any existing files\n" . ansiColor(null)
				. ansiColor('yellow') . "-" . ansiColor(null) . " If you leave this blank it will be installed to \"/usr/share/portalnode-php\".", null, true);

			if (empty($newRootDirectory)) {
				$newRootDirectory = "/usr/share/portalnode-php";
			}
		}

		printLine(ansiColor('green') . "Copying files in progress..." . ansiColor(null) . "\n");
		if (!copyDirectoryContentsRecursively(DIRECTORY_ROOT, $newRootDirectory)) {
			exit(ansiColor('red') . "Couldn't install Portal.Node PHP to \"" . $newRootDirectory . "\"\n" . ansiColor(null));
		}

		$newApplicationDirectory = $newRootDirectory . "/app";
	} else {
		$newRootDirectory = DIRECTORY_ROOT;
		$newApplicationDirectory = DIRECTORY_APPLICATION;
	}

	// Change to the new application directory
	chdir($newApplicationDirectory);

	// Memcached checking and installation
	if (!shellCommandExists("memcached")) {
		if (!isWindows() && prompt("memcached does not exist. Shall we try to install it?")) {
			if (shellCommandExists("apt-get")) {
				passthru('apt-get -y install memcached php5-memcached php5-memcache');
				passthru('/etc/init.d/memcached restart');
			} else if (shellCommandExists("yum")) {
				passthru('yum -y install memcached php-pecl-memcached php-pecl-memcache');
				passthru('service memcached start');
				passthru('chkconfig memcached on');
			} else if (shellCommandExists("up2date")) {
				passthru('up2date memcached php-pecl-memcached php-pecl-memcache');
				passthru('service memcached start');
				passthru('chkconfig memcached on');
			} else {
				printLine(ansiColor('red') . "[ERROR] No package manager found! Please install \"memcached\" manually.\n" . ansiColor(null));
			}
		} else {
			printLine(ansiColor('red') . "Please install \"memcached\" manually.\n" . ansiColor(null));
		}
	}

	if (!isWindows()) {
		if (ENVIRONMENT === "build") {
			// Unix: Make folders and files writeable
			echo `chown -R www-data "$newApplicationDirectory"`;
			echo `chmod go+w $newApplicationDirectory/custom/config/*`;
			echo `chmod go+w $newApplicationDirectory/common/runtime`;
			echo `chmod go+w $newApplicationDirectory/custom/runtime`;
			echo `chmod go+w $newApplicationDirectory/tests/runtime`;
			echo `chmod go+w $newApplicationDirectory/frontend/runtime`;
			echo `chmod go+w $newApplicationDirectory/frontend/www/assets`;
		}

		echo `chmod a+x $newApplicationDirectory/common/yiic`;
	}

	// Copy the custom configuration files from *.dist
	foreach (glob($newApplicationDirectory . "/custom/config/*.dist") as $distFile) {
		$newFile = substr($distFile, 0, -5);

		if (file_exists($newFile)) {
			continue;
		}

		copy($distFile, $newFile);
		chown($newFile, fileowner($distFile));
		chgrp($newFile, filegroup($distFile));
	}

	$ClientMainConfiguration = include "$newApplicationDirectory/custom/config/client_main.php";

	$settings['components']['db'] = array();

	$connectionSuccessful = false;
	if (prompt('Do you want to use a database?')) {
		$connectionSuccessfulOrAbort = false;

		while (!isset($GLOBALS['options']['unattended-installation']) && !$connectionSuccessfulOrAbort) {
			if (empty($GLOBALS['options']['database-connectionstring'])) {
				do {
					$n = ask("What kind of database do you want to use?" . ansiColor(null),
						array(1 => 'MySQL', 2 => 'Postgres', 3 => 'Oracle'));
				} while (!in_array($n, array(1, 2, 3)));

				$host = ask('Database server host (i.e. localhost):');
				$port = ask('Database server port (i.e. 3306, you can also leave it blank):', null, true);

				$database = ask('Database name (Must exist already and be empty):');

				switch ($n) {
					case 1:
						$settings['components']['db']['connectionString'] = "mysql:host=$host" . ($port ? ":$port" : '') . ";dbname=$database";
						break;
					case 2:
						$settings['components']['db']['connectionString'] = "pgsql:host=$host;" . ($port ? "port=$port;" : '') . "dbname=$database";
						break;
					case 3:
						$settings['components']['db']['connectionString'] = "oci:dbname=$host" . ($port ? ":$port" : '') . "/$database;charset=UTF8";
						break;
				}
			} else {
				$settings['components']['db']['connectionString'] = $GLOBALS['options']['database-connectionstring'];
			}

			if (empty($GLOBALS['options']['database-username'])) {
				$settings['components']['db']['username'] = ask('Username:');
			} else {
				$settings['components']['db']['username'] = $GLOBALS['options']['database-username'];
			}

			if (empty($GLOBALS['options']['database-password'])) {
				$settings['components']['db']['password'] = ask('Password:', null, true);
			} else {
				$settings['components']['db']['password'] = $GLOBALS['options']['database-password'];
			}

			$connection = new CDbConnection($settings['components']['db']['connectionString'], $settings['components']['db']['username'], $settings['components']['db']['password']);
			try {
				$connection->setActive(true);
				printLine(ansiColor('green') . 'The connection to the database has been established successfully!' . ansiColor(null));
				$connectionSuccessfulOrAbort = true;
				$connectionSuccessful = true;
			} catch (Exception $e) {
				printLine(ansiColor('red') . '[ERROR] ' . $e->getMessage() . ansiColor(null));

				if (!prompt('Try again?', true, true)) {
					$connectionSuccessfulOrAbort = true;
					printLine("You can configure it manually. Configure the 'db' component settings in client_main.php.");
					ask('Press enter to continue', null, true);
				}
			}
		}
	}

	// ContentSource
	if (prompt('Do you want to use the Dynamic Content Renderer (DCR) (recommended, requires Gentics Content Connector running)?')) {
		$sett = &$settings['modules']['contentSource']['sourceSettings']['DynamicContentSource'];
		$sett['cacheFolder'] = ask('Location of your cache folder (i.e. /usr/share/portalnode-php/DCR/), must be writeable for webserver-user:',
			null, false, "/usr/share/portalnode-php/DCR/");
		$sett['usePersonalisation'] = false;
		$sett['personalisationFields'] = array();
		if (prompt('Will you use personalisation?')) {
			$sett['usePersonalisation'] = true;
			$personalisationFields = ask('Enter personalisation attributes (multivalue) requested by Gentics Content Connector'
				. ' - must be defined in CMS (Tagmap) (i.e. permissions):', null, true);
			$sett['personalisationFields'] = array_map('trim', explode(',', $personalisationFields));
		}
		$settings['modules']['contentSource']['sourceClass'] = 'DynamicContentSource';
	}

	if (prompt('Do you want to use the File System Content Renderer (FSCR) alternatively?')) {
		$sett = &$settings['modules']['contentSource']['sourceSettings']['FileSystemContentSource'];
		$sett['contentFolder'] = ask('Location of your published CMS files (i.e. /usr/share/portalnode-php/FSCR/):'
			, null, false, "/usr/share/portalnode-php/FSCR/");
		$sett['usePersonalisation'] = false;
		$sett['personalisationFields'] = array();
		if (prompt('Will you use personalisation?')) {
			$sett['usePersonalisation'] = true;
			$personalisationFields = ask('Enter personalisation attributes (multivalue) requested by Gentics Content Connector'
				. ' - must be defined in CMS (Tagmap) (i.e. permissions):', null, true);
			if (!empty($personalisationFields)) {
				$sett['personalisationFields'] = array_map('trim', explode(',', $personalisationFields));
			}
		}
		$settings['modules']['contentSource']['sourceClass'] = 'FileSystemContentSource';
	}

	// Memcached
	if (prompt('Do you want to use memcached as caching system (recommended, required for sessions)?')) {
		$settings['components']['cache']['servers']['server1'] = array('host' => '127.0.0.1', 'port' => 11211, 'weight' => 60);
	}

	// Eauth
	$settings['components']['eauth']['services']['standard'] = array(
		'title' => 'Use default login with username and password?'
	);
	if (prompt('Enable Google authentication?')) {
		$settings['components']['eauth']['services']['google'] = array(
			'class' => 'site.common.modules.user.services.GoogleService',
			'title' => 'Use login with Google Open ID?'
		);
	}

	if (prompt('Enable Facebook Authentication (you need a Facebook App)?')) {
		$settings['components']['eauth']['services']['facebook'] = array(
			'title' => 'Use login with Facebook?',
			'class' => 'site.common.modules.user.services.FacebookService',
		);
	}

	// Since the database configuration has its own file now, split it into an extra
	// variable.
	$databaseSettings = $settings['components']['db'];
	unset($settings['components']['db']);

	file_put_contents(CLIENT_CONFIG_LOCATION, "<?php\n\nreturn " . var_export(CMap::mergeArray($ClientMainConfiguration, $settings), true) . ";\n");

	$databaseMainConfiguration = include "$newApplicationDirectory/custom/config/_database.php.dist";
	file_put_contents($newApplicationDirectory . '/custom/config/_database.php',
		"<?php\n\nreturn " . var_export(CMap::mergeArray($databaseMainConfiguration, $databaseSettings), true) . ";\n");

	printLine('Configuration finished! You can find the settings in ' . ansiColor('green') . realpath(CLIENT_CONFIG_LOCATION) . ansiColor(null));

	// Update the client_console.php-file
	/*$myOldConsole = include $newApplicationDirectory . '/custom/config/client_console.php';
	file_put_contents($newApplicationDirectory . '/custom/config/client_console.php',
		'<?php Yii::setPathOfAlias(\'site\', dirname(__FILE__) . DIRECTORY_SEPARATOR . \'..\' . DIRECTORY_SEPARATOR . \'..\'); return ' .
	var_export(CMap::mergeArray($myOldConsole, array('components' => array('db' => $settings['components']['db']))), true) . "\n");*/


	// Copy the client_config to production
	$date = date("YmdHMs");
	copy($newApplicationDirectory . "/custom/config/client_main.php",  $newApplicationDirectory . "/custom/config/client_main_$date.php");
	copy($newApplicationDirectory . "/custom/runtime/client_main.php", $newApplicationDirectory . "/custom/config/client_main.php");

	if ($connectionSuccessful) {
		// Run the YII migrations
		chdir($newApplicationDirectory . "/common");
		printLine('Running database migration...');
		runYiiMigration("php ./yiic.php");
		chdir($newApplicationDirectory . "/scripts");
	} else {
		printLine("Please run the following command if you want to use a database: "
			. ansiColor('green') . $yiicMigrationCommand . ansiColor(null) . " (in scripts/)");
	}

	printLine(ansiColor('green') . "\n[SUCCESS] Installation finished! You can delete this directory now." . ansiColor(null));
}

/**
 * Update a Gentics Portal.Node PHP application instance
 */
function update()
{
	$applicationDirectory = null;

	// Don't mess around with files in development environments
	if (ENVIROMENT === "build") {
		while ($applicationDirectory === null) {
			$applicationDirectory = ask("Please specify the path to the app directory of Gentics Portal.Node PHP (Default: /usr/share/portalnode-php/app)",
				null, true, "/usr/share/portalnode-php/app");

			if (empty($applicationDirectory)) {
				$applicationDirectory = "/usr/share/portalnode-php/app";
			}

			if (!file_exists($applicationDirectory . "/frontend/www/index.php")) {
				printLine(ansiColor("red") . "[ERROR] Folder is not correct. Please try again." . ansiColor(null));
				$applicationDirectory = null;
			}

			$clientConsolePhpPath = $applicationDirectory . "/custom/config/client_console.php";
			if (!file_exists($clientConsolePhpPath)) {
				printLine(ansiColor("red") . "[ERROR] \"" . $clientConsolePhpPath . "\""
					." does not exist. Please create it by copying the client_console.php.dist and specify DB connection parameters in it first". ansiColor(null));

				if (!prompt("Try again?")) {
					exit();
				}
			}
		}

		// Ask if to create a backup
		if (prompt("Shall we make a backup of your current installation (does not backup database)?")) {
			$backupRootDirectory = null;
			while ($backupRootDirectory === null) {
				$backupRootDirectory = ask("Please define the root backup directory (a new sub directory will be created):");

				if (!file_exists($backupRootDirectory) || !is_dir($backupRootDirectory)) {
					printLine(ansiColor("red") . "[ERROR] Directory " . $backupRootDirectory . " is not valid. Please try again." . ansiColor(null));
					$backupRootDirectory = null;
				}
			}

			$backupDirectory = $backupRootDirectory . "/portalnode-php_" . date("y-m-d-H-m-s");
			mkdir($backupDirectory, 0770, true);

			printLine(ansiColor("green") . "Please make a backup of your current Gentics Portal.Node PHP database yourself." . ansiColor(null));
		}

		ask("Press " . ansiColor("green") . "enter" . ansiColor('yellow') . " to start the update now:", null, true);

		// First, delete all folders, except "custom" and "frontend"
		foreach (glob($applicationDirectory . "/*") as $fileName) {
			if (in_array(basename($fileName), array("custom", "frontend"))) {
				rename($fileName, $fileName . "_old");
			}
		}

		// Copy stuff
		copyDirectoryContentsRecursively(DIRECTORY_APPLICATION, $applicationDirectory);
		copyDirectoryContentsRecursively($applicationDirectory . "/custom_old",   $applicationDirectory . "/custom");
		copyDirectoryContentsRecursively($applicationDirectory . "/frontend_old", $applicationDirectory . "/frontend");

		// Delete custom_old and frontend_old
		removeDirectoryRecursively($applicationDirectory . "/custom_old");
		removeDirectoryRecursively($applicationDirectory . "/frontend_old");

		if (!isWindows()) {
			// Set the correct file permissions
			passthru("chmod go+w \"" . $applicationDirectory . "/custom/config/\"*");
			passthru("chmod go+w \"" . $applicationDirectory . "/common/runtime\"");
			passthru("chmod go+w \"" . $applicationDirectory . "/custom/runtime\"");
			passthru("chmod go+w \"" . $applicationDirectory . "/tests/runtime\"");
			passthru("chmod go+w \"" . $applicationDirectory . "/frontend/runtime\"");
			passthru("chmod go+w \"" . $applicationDirectory . "/frontend/www/assets\"");
			passthru("chmod a+x \""  . $applicationDirectory . "/common/yiic\"");

			// Set the right owner and group
			$fileOwner = fileowner($applicationDirectory);
			$fileGroup = filegroup($applicationDirectory);
			passthru("chown -R " . $fileOwner . ":" . $fileGroup . " \"$applicationDirectory\"");
		}
	} else {
		$applicationDirectory = __DIR__;
	}

	runYiiMigration("php " . $applicationDirectory . "/common/yiic.php");

	printLine("\n" . ansiColor('green') . "Update finished." . ansiColor(null));
}

/**
 * Show the help to the user
 */
function help()
{
	printLine("Available command line parameters:");

	registerParameter("unattended-installation",   "Enable the unattended installation - Beware that system packet manager dialogs will still be displayed");
	registerParameter("database-connectionstring", "Installer: Set the database connectionstring for the unattended installation."
		. " Example: mysql:host=localhost;dbname=portalnode-php");
	registerParameter("database-password",         "Installer: Set the database password for the unattended installation");
	registerParameter("database-username",         "Installer: Set the database username for the unattended installation");
	registerParameter("help",                      "Display this help");
	registerParameter("install",                   "Install Gentics Portal.Node PHP");
	registerParameter("install-lampp",             "Experimental: Install a LAMP on Linux Debian Wheezy also during installation");
	registerParameter("update",                    "Update an existing installation");

	foreach($GLOBALS['parameters'] as $keyword => $parameter) {
		printLine(ansiColor('green') . "  --" . str_pad($keyword, 32)
			. ansiColor('yellow') . $parameter['info'] . ansiColor(null));
	}
}

/**
 * Parse the arguments passed to the script and store then in
 * $options as array.
 *
 * @param array $arguments   An array of arguments, you normally want to pass
 *                           the magic PHP variable $argv here
 * @param array $commandList An array of available commands
 * @return string The command (install, update) it one was found, null otherwise
 */
function parseCommandLineArguments($arguments, $commandList, &$options)
{
	$command = null;

	for ($i = 1; $i < count($arguments); $i++) {
		$argument = $arguments[$i];

		if (substr($argument, 0, 2) === "--") {
			$option = strtolower(substr($argument, 2));

			if (array_key_exists($option, $commandList)) {
				$command = $option;
			}

			$tokens = explode("=", $option, 2);
			if (count($tokens) === 2) {
				$options[$tokens[0]] = trim($tokens[1], "\"' \t\n\r\0\x0B");
			} else {
				$options[$option] = true;
			}
		}
	}

	return $command;
}

/**
 * Register a new command line parameter
 *
 * @param string $keyword Keyword (without -- in front)
 * @param string $info    Info text
 */
function registerParameter($keyword, $info) {
	$parameters = &$GLOBALS['parameters'];

	$parameters[$keyword] = array(
		'info' => $info
	);
}

/**
 * Return the current Gentics Portal.Node version or *dev*
 *
 * @return string version
 */
function getVersion()
{
	if (ENVIRONMENT === "dev") {
		$path = "src/main/resources";
	} else {
		$path = ".";
	}

	$versionTxt = null;
	if (file_exists($path . "/VERSION.txt")) {
		$versionTxt = file_get_contents($path . "/VERSION.txt");
	}

	if (!$versionTxt || substr($versionTxt, 0, 1) === '$') {
		return "*dev*";
	}

	return "v" . $versionTxt;
}



/**************************
 * Directly executed code
 **************************/

$parameters = array();

$commandList = array(
	'install' => "install",
	'update'  => "update",
	'help'    => "help",
	'unattended-installation' => "install"
);

$ansiColors = array(
	'red'      => '01;31',
	'green'    => '0;32',
	'purple'   => '0;35',
	'yellow'   => '1;33'
);



// Program start
chdir(__DIR__); // __DIR__ was introduced in PHP 5.3.0

$options = array();
$command = parseCommandLineArguments($argv, $commandList, $options);

// PHP version check
$phpVersion = phpversion();
if (strcmp($phpVersion, "5.3.0") < 0) {
	// PHP version is equal or greater than 5.3.0
	die("Your PHP version is " . $phpVersion . ". You need at least 5.3.0\n");
}

if (file_exists("app")) {
	define('DIRECTORY_ROOT',        realpath('.'));
	define('DIRECTORY_APPLICATION', DIRECTORY_ROOT . '/app');
	define('ENVIRONMENT',           'build');
} elseif (file_exists("../portalnode-php-app")) {
	define('DIRECTORY_ROOT',        realpath('..'));
	define('DIRECTORY_APPLICATION', DIRECTORY_ROOT . '/portalnode-php-app');
	define('ENVIRONMENT',           'dev');
} else {
	die("GPN PHP application directory not found\n");
}

printLine(ansiColor('green') . "# Gentics Portal.Node PHP " . getVersion() . " Command Line Tool (specify --help for more options)" . ansiColor(null) . "\n");

define('CLIENT_CONFIG_LOCATION', DIRECTORY_APPLICATION . '/custom/runtime/client_main.php');

require DIRECTORY_APPLICATION . '/framework/yii.php';
require DIRECTORY_APPLICATION . '/framework/db/CDbConnection.php';

while (empty($command)) {
	$answer = ask("What do you want to do?" . ansiColor(null),
		array(
			1 => 'Install Gentics Portal.Node PHP',
			2 => 'Update an existing installation',
			3 => 'Show help',
			4 => 'Exit'
		)
	);

	switch ($answer) {
	case 1:
		$command = "install";
		break;
	case 2:
		$command = "update";
		break;
	case 3:
		$command = "help";
		break;
	case 4:
		exit();
	}
}

// Execute the command
$commandList[$command]();
